package com.qucheng.game.data.oldsystem.sink;

import com.qucheng.game.data.oldsystem.dao.mapper.BaseMapper;
import com.qucheng.game.data.oldsystem.pojo.MysqlConfigParam;
import com.qucheng.game.data.oldsystem.pojo.TransportMap;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import io.debezium.data.Envelope;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.apache.ibatis.mapping.Environment;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.apache.ibatis.transaction.jdbc.JdbcTransactionFactory;

import javax.sql.DataSource;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public class CdcMysqlTablesSink extends RichSinkFunction<TransportMap> {
    private final Map<String, String> tableNameMap;
    private final MysqlConfigParam mysqlConfigParam;
    protected SqlSessionFactory sqlSessionFactory;

    public CdcMysqlTablesSink(MysqlConfigParam mysqlConfigParam) {
        this(null, mysqlConfigParam);
    }

    public CdcMysqlTablesSink(Map<String, String> tableNameMap, MysqlConfigParam mysqlConfigParam) {
        this.tableNameMap = tableNameMap == null ? Collections.emptyMap() : tableNameMap;
        this.mysqlConfigParam = mysqlConfigParam;
    }

    @Override
    public void open(Configuration configuration) {
        HikariConfig config = new HikariConfig();
        config.setDriverClassName(mysqlConfigParam.getDriverClassName());
        config.setJdbcUrl(mysqlConfigParam.getUrl());
        config.setUsername(mysqlConfigParam.getUsername());
        config.setPassword(mysqlConfigParam.getPassword());
        //连接池内保留的最少连接数
        config.setMinimumIdle(1);
        DataSource dataSource = new HikariDataSource(config);

        Environment mybatisEnv = new Environment("mysql-sink", new JdbcTransactionFactory(), dataSource);

        org.apache.ibatis.session.Configuration mybatisConfig = new org.apache.ibatis.session.Configuration(mybatisEnv);
        mybatisConfig.addMapper(BaseMapper.class);
        sqlSessionFactory = new SqlSessionFactoryBuilder().build(mybatisConfig);
    }

    @Override
    public void invoke(TransportMap value, Context context) {
        if (Envelope.Operation.TRUNCATE == value.getOperation()) {
            return;
        }
        if (value.getPrimaryKeys() == null || value.getPrimaryKeys().isEmpty()) {
            return;
        }
        String tableName = tableNameMap.get(value.getTableName());
        tableName = StringUtils.isBlank(tableName) ? value.getTableName() : tableName;
        List<String> primaryKeys = value.getPrimaryKeys();
        Map<String, Object> data = value.getAfter();
        if (Envelope.Operation.DELETE == value.getOperation()) {
            // 删除
            Map<String, Object> delParamMap = primaryKeys.stream().collect(Collectors.toMap(Function.identity(), data::get));
            try (SqlSession session = sqlSessionFactory.openSession(true)) {
                BaseMapper mapper = session.getMapper(BaseMapper.class);
                mapper.delById(tableName, delParamMap);
            }
        } else {
            // insertOrUpdate
            try (SqlSession session = sqlSessionFactory.openSession(true)) {
                BaseMapper mapper = session.getMapper(BaseMapper.class);
                mapper.saveOrUpdate(tableName, primaryKeys, data);
            }
        }
    }

    @Override
    public void close() {
    }
}
